@@ -1,10 +1,20 @@
 """Test serializing ecdsa keys"""
 from __future__ import absolute_import, division, unicode_literals
+
 import unittest
 
+import pytest
+from helper import SkippableTest
+
 import jsonpickle
 
-from helper import SkippableTest
+
+@pytest.fixture(scope='module', autouse=True)
+def gmpy_extension():
+    """Initialize the gmpy extension for this test module"""
+    jsonpickle.ext.gmpy.register_handlers()
+    yield  # control to the test function.
+    jsonpickle.ext.gmpy.unregister_handlers()
 
 
 class EcdsaTestCase(SkippableTest):
