@@ -1,24 +1,21 @@
 from __future__ import absolute_import, division, unicode_literals
+
 import array
-import enum
 import collections
 import datetime
 import decimal
+import enum
 import re
 import threading
-import pytz
-
-import jsonpickle
-from jsonpickle import compat
-from jsonpickle import handlers
-from jsonpickle import tags
-from jsonpickle import util
-from jsonpickle.compat import queue, PY2, PY3_ORDERED_DICT
 
 import pytest
-
+import pytz
 from helper import SkippableTest
 
+import jsonpickle
+from jsonpickle import compat, handlers, tags, util
+from jsonpickle.compat import queue
+
 
 class Thing(object):
     def __init__(self, name):
@@ -108,15 +105,15 @@ class SetSubclass(set):
     pass
 
 
+def func(x):
+    return x
+
+
 class ThingWithFunctionRefs(object):
     def __init__(self):
         self.fn = func
 
 
-def func(x):
-    return x
-
-
 class ThingWithQueue(object):
     def __init__(self):
         self.child_1 = queue.Queue()
@@ -522,22 +519,15 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertTrue(decoded == u1)
         self.assertTrue(isinstance(decoded, compat.ustr))
 
-        # bytestrings are wrapped in PY3 but in PY2 we try to decode first
+        # bytestrings are wrapped in py 3
         encoded = self.pickler.flatten(b1)
-        if PY2:
-            self.assertEqual(encoded, u1)
-            self.assertTrue(isinstance(encoded, compat.ustr))
-        else:
-            self.assertNotEqual(encoded, u1)
-            encoded_ustr = util.b64encode(b'foo')
-            self.assertEqual({tags.B64: encoded_ustr}, encoded)
-            self.assertTrue(isinstance(encoded[tags.B64], compat.ustr))
+        self.assertNotEqual(encoded, u1)
+        encoded_ustr = util.b64encode(b'foo')
+        self.assertEqual({tags.B64: encoded_ustr}, encoded)
+        self.assertTrue(isinstance(encoded[tags.B64], compat.ustr))
         decoded = self.unpickler.restore(encoded)
         self.assertTrue(decoded == b1)
-        if PY2:
-            self.assertTrue(isinstance(decoded, compat.ustr))
-        else:
-            self.assertTrue(isinstance(decoded, bytes))
+        self.assertTrue(isinstance(decoded, bytes))
 
         # bytestrings that we can't decode to UTF-8 will always be wrapped
         encoded = self.pickler.flatten(b2)
@@ -549,21 +539,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(decoded, b2)
         self.assertTrue(isinstance(decoded, bytes))
 
-    def test_backcompat_bytes_quoted_printable(self):
-        """Test decoding bytes objects from older jsonpickle versions"""
-
-        b1 = b'foo'
-        b2 = b'foo\xff'
-
-        # older versions of jsonpickle used a quoted-printable encoding
-        expect = b1
-        actual = self.unpickler.restore({tags.BYTES: 'foo'})
-        self.assertEqual(expect, actual)
-
-        expect = b2
-        actual = self.unpickler.restore({tags.BYTES: 'foo=FF'})
-        self.assertEqual(expect, actual)
-
     def test_nested_objects(self):
         obj = ThingWithTimedeltaAttribute(99)
         flattened = self.pickler.flatten(obj)
@@ -607,12 +582,6 @@ class AdvancedObjectsTestCase(SkippableTest):
             obj = array.array(typecode, (1, 2, 3))
             self._test_array_roundtrip(obj)
 
-    def test_array_handler_python2(self):
-        """Python2 allows the "c" byte/char typecode"""
-        if PY2:
-            obj = array.array('c', bytes('abcd'))
-            self._test_array_roundtrip(obj)
-
     def test_exceptions_with_arguments(self):
         """Ensure that we can roundtrip Exceptions that take arguments"""
         obj = ExceptionWithArguments('example')
@@ -696,27 +665,6 @@ def test_defaultdict_and_things_roundtrip_simple_lambda():
     assert decoded[1] is decoded[2]
 
 
-def test_defaultdict_subclass_with_self_as_default_factory():
-    """Serialize a defaultdict subclass with self as its default factory"""
-    cls = ThingWithSelfAsDefaultFactory
-    tree = cls()
-    newtree = _test_defaultdict_tree(tree, cls)
-    assert type(newtree['A'].default_factory) == cls
-    assert newtree.default_factory is newtree
-    assert newtree['A'].default_factory is newtree['A']
-    assert newtree['Z'].default_factory is newtree['Z']
-
-
-def test_defaultdict_subclass_with_class_as_default_factory():
-    """Serialize a defaultdict with a class as its default factory"""
-    cls = ThingWithClassAsDefaultFactory
-    tree = cls()
-    newtree = _test_defaultdict_tree(tree, cls)
-    assert newtree.default_factory is cls
-    assert newtree['A'].default_factory is cls
-    assert newtree['Z'].default_factory is cls
-
-
 def _test_defaultdict_tree(tree, cls):
     tree['A']['B'] = 1
     tree['A']['C'] = 2
@@ -741,6 +689,27 @@ def _test_defaultdict_tree(tree, cls):
     return newtree
 
 
+def test_defaultdict_subclass_with_self_as_default_factory():
+    """Serialize a defaultdict subclass with self as its default factory"""
+    cls = ThingWithSelfAsDefaultFactory
+    tree = cls()
+    newtree = _test_defaultdict_tree(tree, cls)
+    assert type(newtree['A'].default_factory) == cls
+    assert newtree.default_factory is newtree
+    assert newtree['A'].default_factory is newtree['A']
+    assert newtree['Z'].default_factory is newtree['Z']
+
+
+def test_defaultdict_subclass_with_class_as_default_factory():
+    """Serialize a defaultdict with a class as its default factory"""
+    cls = ThingWithClassAsDefaultFactory
+    tree = cls()
+    newtree = _test_defaultdict_tree(tree, cls)
+    assert newtree.default_factory is cls
+    assert newtree['A'].default_factory is cls
+    assert newtree['Z'].default_factory is cls
+
+
 def test_posix_stat_result():
     """Serialize a posix.stat() result"""
     try:
@@ -777,8 +746,6 @@ def test_broken_repr_dict_key():
 
 def test_ordered_dict_python3():
     """Ensure that we preserve dict order on python3"""
-    if not PY3_ORDERED_DICT:
-        return
     # Python3.6+ preserves dict order.
     obj = {'z': 'Z', 'x': 'X', 'y': 'Y'}
     clone = jsonpickle.decode(jsonpickle.encode(obj))
